<?php

namespace App\Models;

use DateTime;
use Illuminate\Database\Eloquent\Concerns\HasTimestamps;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasMany;

/**
 * Class Evaluation.
 *
 * @property int                     $id
 * @property DateTime                $createdAt
 * @property DateTime                $updatedAt
 * @property DangerLevel[]|iterable  $dangerLevels
 * @property DangerLevel[]|iterable  $danger_levels
 * @property MeasureLevel[]|iterable $measureLevels
 * @property MeasureLevel[]|iterable $measure_levels
 * @property int                     $status
 * @property Organization            $organization
 * @property int                     $organization_id
 * @property int                     $current_step
 * @property int                     $current_step_status
 * @property string                  $author
 * @property string                  $updated_by
 */
class Evaluation extends Model
{
    use HasFactory;
    use HasTimestamps;

    public const STATUS_ONGOING = 1;
    public const STATUS_DONE = 2;

    public const STEP_DANGERS = 1;
    public const STEP_MEASURES = 2;
    public const STEP_RECAP = 3;
    public const STEP_ACTIONS = 4;
    public const STEP_MATURITY = 5;
    public const STEP_RESULTS = 6;

    public const STATUS = [
        self::STATUS_ONGOING => 'En cours',
        self::STATUS_DONE => 'Terminé',
    ];

    public function dangerLevels(): BelongsToMany
    {
        return $this->belongsToMany(DangerLevel::class, 'danger_level_evaluation');
    }

    public function measureLevels(): HasMany
    {
        return $this->hasMany(MeasureLevel::class);
    }

    public function organization(): BelongsTo
    {
        return $this->belongsTo(Organization::class);
    }
}
