<?php

namespace Database\Seeders;

use App\Models\Danger;
use App\Models\Evaluation;
use App\Models\Measure;
use App\Models\MeasureLevel;
use App\Models\Organization;
use App\Models\User;
use Illuminate\Database\Seeder;

class OrganizationSeeder extends Seeder
{
    /**
     * Run the database seeds.
     *
     * @return void
     */
    public function run()
    {
        Organization::factory(5)->create()->each(function (Organization $org) {
            $org->users()->saveMany(User::factory(5)->make());
            $org->users()->saveMany(User::factory(1)->make(['role' => User::ROLE_MANAGER]));
            $org->users()->saveMany(User::factory(1)->make(['role' => User::ROLE_USER]));
            $org->evaluations()->saveMany(Evaluation::factory(5)->make());
            $org->evaluations()->saveMany(Evaluation::factory(2)->make(['current_step' => 4]));
            $org->evaluations()->saveMany(Evaluation::factory(2)->make(['current_step' => 5]));
            $org->evaluations()->saveMany(Evaluation::factory(2)->make([
                'current_step' => 5,
                'reference' => env('REFERENTIEL_VERSION'),
            ]));
            $org->evaluations()->saveMany(Evaluation::factory(2)->make(['current_step' => 1]));

            foreach ($org->evaluations as $eval) {
                if ($eval->current_step < Evaluation::STEP_RESULTS) {
                    $eval->status = Evaluation::STATUS_ONGOING;
                } else {
                    $eval->status = Evaluation::STATUS_DONE;
                }
                $eval->save();
                if ($eval->current_step >= Evaluation::STEP_DANGERS) {
                    $dangers = Danger::all();

                    $dangerLevels = $dangers->map(function (Danger $danger) {
                        return $danger->levels()->inRandomOrder()->first();
                    });

                    $eval->dangerLevels()->saveMany($dangerLevels);
                }

                if ($eval->current_step >= Evaluation::STEP_MEASURES) {
                    $measures = Measure::all();
                    $evalMeasures = $measures->map(function (Measure $m) {
                        return MeasureLevel::factory()->make(['measure_id' => $m->id]);
                    });
                    $eval->MeasureLevels()->saveMany($evalMeasures);
                }
            }
        });

        // Create inactive org with active user
        Organization::factory(1)->create(['active' => false])->each(function (Organization $org) {
            $org->users()->saveMany(User::factory(1)->make(['role' => User::ROLE_MANAGER]));
            $org->users()->saveMany(User::factory(5)->make(['active' => true, 'role' => User::ROLE_USER]));
        });
        // Create active org with inactive user
        Organization::factory(1)->create(['active' => true])->each(function (Organization $org) {
            $org->users()->saveMany(User::factory(1)->make(['role' => User::ROLE_MANAGER]));
            $org->users()->saveMany(User::factory(5)->make(['active' => false, 'role' => User::ROLE_USER]));
        });
    }
}
